/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/Services.jsm");

var EXPORTED_SYMBOLS = ["ABPIntegration"];
var ABPInstalled = false;

// ABP 2.0 and older
var ABPPrivate = Cc["@adblockplus.org/abp/private;1"];
if(ABPPrivate)
{
	var privateURL = ABPPrivate.getService(Ci.nsIURI);
	Cu.import(privateURL.spec + "Matcher.jsm");
	Cu.import(privateURL.spec + "FilterClasses.jsm");
	Cu.import(privateURL.spec + "FilterStorage.jsm");
	Cu.import(privateURL.spec + "SubscriptionClasses.jsm");
	Cu.import(privateURL.spec + "Synchronizer.jsm");
	Cu.import(privateURL.spec + "ContentPolicy.jsm");
	Cu.import(privateURL.spec + "Utils.jsm");
	Cu.import(privateURL.spec + "Prefs.jsm");
	try
	{
		Cu.import(privateURL.spec + "FilterNotifier.jsm");
	}
	catch(e)
	{
	}

	ABPInstalled = true;
}

// ABP 2.1+ and Adblock Edge
function require(/**String*/ module)
{
	let result = {};
	result.wrappedJSObject = result;
	Services.obs.notifyObservers(result, "adblockplus-require", module);
	if(!result.exports)
	{
		// Try Adblock Edge
		Services.obs.notifyObservers(result, "adblockedge-require", module);
		if(!result.exports)
		{
			return null;
		}
	}
	return result.exports;
}
if(require("contentPolicy"))
{
	var {Policy} = require("contentPolicy");
	var {Filter, InvalidFilter, CommentFilter, ActiveFilter, RegExpFilter,
		 BlockingFilter, WhitelistFilter, ElemHideFilter} = require("filterClasses");
	var {FilterNotifier} = require("filterNotifier");
	var {FilterStorage, PrivateBrowsing} = require("filterStorage");
	var {IO} = require("io");
	var {defaultMatcher, Matcher, CombinedMatcher} = require("matcher");
	var {Prefs} = require("prefs");
	var {RequestNotifier} = require("requestNotifier");
	var {Subscription, SpecialSubscription, RegularSubscription,
		 ExternalSubscription, DownloadableSubscription} = require("subscriptionClasses");
	var {Synchronizer} = require("synchronizer");
	var {UI} = require("ui");
	var {Utils} = require("utils");
	ABPInstalled = true;
}

if(ABPInstalled)
{
	var publicURL = Cc["@adblockplus.org/abp/public;1"].getService(Ci.nsIURI);
	Cu.import(publicURL.spec);
}

const ABPIntegration = 
{
	loaded: false,
	installed: false,
	observers: [],
	runAsync: null,

	get enabled()
	{
		return this.installed && Prefs && Prefs.enabled;
	},

	init: function()
	{
		if(this.loaded) return;

		this.loaded = true;
		this.installed = ABPInstalled;

		this.tldService = Cc["@mozilla.org/network/effective-tld-service;1"]
                              .getService(Ci.nsIEffectiveTLDService);

		if(this.installed)
		{
			this.runAsync = Utils.runAsync;

			// ABP 1.3.5
			if(FilterStorage.addObserver)
			{
				var self = this;

				// ABP 1.3.5
				FilterStorage.addObserver(function(action, items)
				{
					if(!items) return;

					for(var i = 0; i < items.length; i++)
					{
						switch(action)
						{
							case 'subscriptions add':
							case 'subscriptions enable':
								self.handleSubscriptionAction(items[i], 'add');
								break;
							case 'subscriptions update':
								self.handleSubscriptionAction(items[i], 'update');
								break;
							case 'subscriptions remove':
							case 'subscriptions disable':
								self.handleSubscriptionAction(items[i], 'remove');
								break;
							default:
								return;
						}
					}
				});
			}
			// ABP 2.0
			else if(FilterNotifier && FilterNotifier.addListener)
			{
				var self = this;

				FilterNotifier.addListener(function(action, item, param1, param2, param3)
				{
					switch (action)
					{
					case 'subscription.updated':
						self.handleSubscriptionAction(item, 'add');
						break;
					case 'subscription.removed':
						self.handleSubscriptionAction(item, 'remove');
						break;
					case 'subscription.disabled':
						var action = item.disabled ? 'remove' : 'add';
						self.handleSubscriptionAction(item, action);
						break;
					default:
						return;
					}
				});
			}
		}
	},

	addSubscriptionObserver: function(callback)
	{
		this.observers.push(callback);
	},

	getSubscriptions: function()
	{
		subscriptions = [];

		if(!this.installed || !FilterStorage.subscriptions) return subscriptions;

		for each (let subscription in FilterStorage.subscriptions)
		{
			var subscr = this.convertSubscription(subscription);

			if(subscr)
			{
				subscriptions.push(subscr);
			}
		}

		return subscriptions;
	},

	convertSubscription: function(subscription)
	{
		if (!(subscription instanceof DownloadableSubscription))
				return null;

		var result = {title: subscription.title, 
		              url: subscription.url, 
		              filters: []};

		if (subscription.filters.length)
		{
			for(var i = 0; i < subscription.filters.length; i++)
			{
				result.filters.push({text: subscription.filters[i].text, 
				                     disabled: subscription.filters[i].disabled});
			}
		}
		else
		{
			return null;
		}

		return result;

	},

	handleSubscriptionAction: function(subscription, action)
	{
		if (!(subscription instanceof DownloadableSubscription))
				return;

		var subscr = this.convertSubscription(subscription);

		if(subscr)
		{
			for(var i = 0; i < this.observers.length; i++)
			{
				this.observers[i].call(this, subscr, action);
			}
		}

	},

	addSubscription: function(url, title)
	{
		if(!this.installed)
		{
			return false;
		}

		let subscription = Subscription.fromURL(url);
		if (!subscription)
			return;

		subscription.title = title;
		subscription.autoDownload = true;
		subscription.disabled = false;
		FilterStorage.addSubscription(subscription);
		if (subscription instanceof DownloadableSubscription && !subscription.lastDownload)
			Synchronizer.execute(subscription);
		FilterStorage.saveToDisk();
		return true;
	},

	popupsSupported: function()
	{
		if(!this.installed || Policy.type.POPUP)
		{
			return true;
		}

		return false;
	},

	isThirdParty: function(uri, opener)
	{
		try
		{
			return this.tldService.getBaseDomain(uri) != this.tldService.getBaseDomain(opener);
		}
		catch (e)
		{
			return uri.host != opener.host;
		}
	},

	isListed: function(uri, opener, type)
	{
		if(!this.installed || !Prefs.enabled)
		{
			return false;
		}

		return defaultMatcher.matchesAny(uri.spec, type, opener.host, this.isThirdParty(uri, opener));
	},

	addPattern: function(pattern)
	{
		if(!this.installed)
		{
			return false;
		}

		AdblockPlus.addPatterns([pattern], 1);
		return true;
	},

	isWhitelisted: function(uri)
	{
		var match = null;

		// Check if window is whitelisted in Adblock
		if ((match = this.isListed(uri, uri, "DOCUMENT")) && (match instanceof WhitelistFilter))
		{
			return true;
		}

		return false;
	},

	isBlacklisted: function(uri, opener)
	{
		var match = null;

		if(match = this.isListed(uri, opener, "POPUP"))
		{
			if(match instanceof WhitelistFilter)
			{
				return false;
			}
			else
			{
				FilterStorage.increaseHitCount(match);
				return true;
			}
		}

		return false;
	}

};

ABPIntegration.init();