/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Jesse Hakanen.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;
const Cu = Components.utils;

var EXPORTED_SYMBOLS = ["SiteRule", "RuleSubscription"];

function SiteRule(site, action, winTypes, closeAsync)
{
	this.site = site;
	this.action = action;
	this.winTypes = winTypes;
	this.enabled = true;
	this.filter = null;
	this.closeAsync = closeAsync;
}

SiteRule.ALLOW = 1;
SiteRule.BLOCK = 2;
SiteRule.CHECK = 4;
SiteRule.CHECK_WINDOWS = 1;
SiteRule.CHECK_TABS = 2;
SiteRule.CHECK_REGULAR = 4;
SiteRule.CHECK_DIALOGS = 8;

SiteRule.actionMap = {
	"ALLOW" : SiteRule.ALLOW,
	"BLOCK" : SiteRule.BLOCK,
	"CHECK" : SiteRule.CHECK
};

SiteRule.checkMap = {
	"WINDOWS" : SiteRule.CHECK_WINDOWS,
	"TABS" : SiteRule.CHECK_TABS,
	"LINKS": SiteRule.CHECK_REGULAR,
	"ALERTS": SiteRule.CHECK_DIALOGS
};

SiteRule.prototype = 
{
	get isValid()
	{
		return (typeof(this.site) != 'undefined') && this.site.length > 0;
	},

	get site()
	{
		return this._site;
	},

	set site(site)
	{
		if(typeof site == "undefined" || site.length <= 0)
		{
			return;
		}

		this._site = site;
		this.pattern = this.createRegExp(site, '/');
	},

	clear: function()
	{
		this._site = "";
	},

	matches: function(str)
	{
		if(!this.enabled)
		{
			return false;
		}

		if(str.match(this.pattern))
		{
			return true;
		}

		return false;
	},

	hasWindowType: function(type)
	{
		return Boolean(this.winTypes & type);
	},

	addWindowType: function(type)
	{
		this.winTypes |= type;
	},

	createRegExp: function(str, delimiter)
	{
		var pattern = str.replace(new RegExp('[.\\\\+?\\[\\^\\]$(){}=!<>|:\\'+(delimiter || '')+'-]', 'g'), '\\$&');

		// Make *.domain.com also match domain.com
		pattern = pattern.replace(/^\*\\\./, '((.+?)\\.)?');

		// Make domain.com also match www.domain.com
		if(!pattern.match(/^www\\\./i))
		{
			pattern = '(www\\.)?' + pattern;
		}

		pattern = '^' + pattern.replace(/\*/g, '(.*?)') + '$'
		return new RegExp(pattern, 'i');
	},

	toString: function()
	{
		var str = this.site + ";";

		str += (this.enabled ? 1 : 0) + ";";

		for(i in SiteRule.actionMap)
		{
			if(SiteRule.actionMap[i] == this.action)
			{
				str += i;
				break;
			}
		}

		str += ";";

		for(var i in SiteRule.checkMap)
		{
			if(this.winTypes & SiteRule.checkMap[i])
			{
				str += i + ",";
			}
		}

		str = str.replace(/([,;]+)$/g, '');

		str += ";" + (this.closeAsync ? 1 : 0);

		return str;
	},

	fromString: function(str)
	{
		var parts = str.split(';');

		if(parts.length < 3)
		{
			return false;
		}

		this.site = parts[0];
		this.action = SiteRule.BLOCK;
		this.enabled = Boolean(parseInt(parts[1]));
		this.winTypes = 0;
		this.closeAsync = false;

		if(SiteRule.actionMap[parts[2]])
		{
			this.action = SiteRule.actionMap[parts[2]];
		}

		if(parts.length >= 4)
		{
			var checkParts = parts[3].split(',');

			for(var i = 0; i < checkParts.length; i++)
			{
				if(SiteRule.checkMap[checkParts[i]])
				{
					this.winTypes |= SiteRule.checkMap[checkParts[i]];
				}
			}
		}

		if(parts.length >= 5)
		{
			this.closeAsync = Boolean(parseInt(parts[4]));
		}

		return true;
	},

	overLaps: function(other)
	{
		if(this.site != other.site)
		{
			return false;
		}

		if(this.winTypes & other.winTypes)
		{
			return true;
		}

		if(this.action == other.action)
		{
			return true;
		}

		return false;
	}
};

function RuleSubscription(title)
{
	this.title = title;
	this.rules = [];
}

RuleSubscription.RULE_ID = "PAR:";

RuleSubscription.isRuleString = function(str)
{
	return (str.charAt(0) == '!' && 
		    str.substr(1, RuleSubscription.RULE_ID.length) == RuleSubscription.RULE_ID);
}

RuleSubscription.prototype = 
{
	title: null,
	rules: [],

	clear: function()
	{
		this.rules.length = 0;
	},

	addRule: function(rule)
	{
		this.rules.push(rule);
	},

	addRuleFromFilter: function(filter)
	{
		var ruleStr = filter.text.substr(RuleSubscription.RULE_ID.length + 1).replace(/^\s+|\s+$/g, '');
		var rule = new SiteRule;
		var action = SiteRule.BLOCK;
		rule.filter = filter.text;
		rule.enabled = !filter.disabled;

		if(ruleStr.substr(0, 2) == '@@')
		{
			action = SiteRule.ALLOW;
			ruleStr = ruleStr.substr(2);
		}

		rule.action = action;

		var parts = ruleStr.split('$');
		rule.site = parts[0];

		if(parts[1])
		{
			var wintypes = parts[1].split(',');

			for(var i = 0; i < wintypes.length; i++)
			{
				switch(wintypes[i].toLowerCase())
				{
					case 'tabs':
						rule.addWindowType(SiteRule.CHECK_TABS);
						break;
					case 'windows':
						rule.addWindowType(SiteRule.CHECK_WINDOWS);
						break;
					case 'links':
						rule.addWindowType(SiteRule.CHECK_REGULAR);
						break;
					case 'alerts':
						rule.addWindowType(SiteRule.CHECK_DIALOGS);
						break;
				}
			}
		}
		else
		{
			rule.addWindowType(SiteRule.CHECK_WINDOWS);
		}

		if(rule.isValid)
		{
			this.addRule(rule);
		}
	}

};